------------------[[변수 선언]]------------------

local Players = game:GetService("Players") -- GetService로 통해 Players를 저장한 변수
local firstSpawn -- firstSpawn 변수


---------[[플레이어가 SpawnLocation을 밟았을 때, 플레이어의 RespawnLocation 속성에 최근에 밟은 SpawnLocation을 지정하는 함수]]---------

local function addSpawn(spawnLocation) -- addSpawn 함수와 매개변수인 spawnLocation
	spawnLocation.Touched:Connect(function(hit) -- spawnLocation로 지정된 객체에 다른 객체가 닿았을 때, Touched 이벤트로 실행되는 함수
		local character = hit:FindFirstAncestorOfClass("Model") -- SpawnLocation에 닿은 신체 부위 파트의 부모(모델)를 character 변수에 지정한다.
		if character then -- 캐릭터이라면 실행되는 조건문
			local player = Players:GetPlayerFromCharacter(character) -- 플레이어의 캐릭터를 player 변수로 지정한다.
			if player and player.RespawnLocation ~= spawnLocation then -- 플레이어의 RespawnLocation이 최근에 밟은 SpawnLocation이 아닐 때
				local humanoid = character:FindFirstChildOfClass("Humanoid") -- 캐릭터의 Humanoid를 humanoid 변수로 지정한다.
				if humanoid and humanoid:GetState() ~= Enum.HumanoidStateType.Dead then -- 캐릭터가 죽지 않았는지 확인한다.
					player.RespawnLocation = spawnLocation -- 플레이어의 RespawnLocation 속성에 spawnLocation 변수를 지정한다.
				end
			end
		end
	end)
end


---------[[맵에 있는 SpawnLocation을 descendant 변수로 지정해서 addSpawn() 함수의 매개변수로 지정하는 코드]]---------

for _, descendant in pairs(workspace:GetDescendants()) do -- workspace의 모든 자식 객체를 찾는 for문
	if descendant:IsA("SpawnLocation") then -- workspace의 모든 자식 객체 중 SpawnLocation 객체이면 실행되는 조건문
		if descendant.Name == "FirstSpawn" then -- SpawnLocation의 이름이 FirstSpawn이라면 실행되는 조건문
			firstSpawn = descendant -- firstSpawn 변수에 descendant를 지정한다.
		end
		addSpawn(descendant) -- FirstSpawn을 제외하고 다른 SpawnLocation은 addSpawn 함수의 매개변수인 descendant에 SpawnLocation을 지정되며 addSpawn 함수가 실행된다.
	end
end


---------[[새로운 플레이어가 접속하거나 이미 접속한 플레이어들을 찾을 때 실행되는 코드]]---------

local function playerAdded(player) -- 새로운 플레이어가 접속하면 실행되는 함수
	player.RespawnLocation = firstSpawn -- 플레이어의 RespawnLocation에 firstspawn 변수에 지정된 객체로 지정된다.
end

for _, player in pairs(Players:GetPlayers()) do -- 이미 접속한 플레이어들을 Players 안에서 찾는 for문
	playerAdded(player) -- 플레이어를 playerAdded 함수의 매개변수로 지정한다.
end

Players.PlayerAdded:Connect(playerAdded) -- 새로운 플레이어가 게임에 접속하면, playerAdded 함수가 실행된다.